//-----------------------------------------------------------------------------
// File: textures.cpp
//
// Desc: use the simplest hlsl vertex and pixel shader code to texture a few triangles
//	     good introduction for how to use hlsl in direct3d apps
//
// author : adrien.auclair
//	        adrien.auclair@club-internet.fr
// date   : 21/09/2004
//-----------------------------------------------------------------------------
#define D3D_DEBUG_INFO

#include <Windows.h>
#include <mmsystem.h>
#include <d3dx9.h>
#include <dxerr9.h>
#include <assert.h>

#include "util.h"

#define SAFE_RELEASE(x) { if (x!=NULL) { x->Release(); x=NULL; } }

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
LPDIRECT3D9             g_pD3D                 = NULL; // Used to create the D3DDevice
LPDIRECT3DDEVICE9       g_pd3dDevice           = NULL; // Our rendering device
LPDIRECT3DVERTEXBUFFER9 g_pVB_Walls            = NULL; // Buffer to hold vertices for the wall
LPDIRECT3DVERTEXBUFFER9 g_pVB_Floor            = NULL; // Buffer to hold vertices for the floor
LPDIRECT3DVERTEXBUFFER9 g_pVB_Torus            = NULL; // Buffer to hold vertices for the torus
LPDIRECT3DTEXTURE9      g_pTexture             = NULL; // The wall texture
LPDIRECT3DTEXTURE9      g_pTextureFloor        = NULL; // The floor texture
LPDIRECT3DTEXTURE9      g_pWallsNormalMap      = NULL; // The walls normal map
LPDIRECT3DTEXTURE9      g_pFloorNormalMap      = NULL; // The floor normal map

LPDIRECT3DTEXTURE9      g_pCrossTexture        = NULL; // The texture of crosses for the torus
LPDIRECT3DTEXTURE9      g_pTorusNormalMap      = NULL; // The normal map of the crosses for the torus

LPDIRECT3DVERTEXSHADER9      m_pVertexShader      = NULL;  // vertex shader
LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclaration = NULL;  // vertex shader input data type
LPD3DXCONSTANTTABLE          m_pVertexConstants   = NULL;  // array of vertex shader constants

LPDIRECT3DVERTEXSHADER9      m_pTorusVertexShader      = NULL;  // vertex shader
LPDIRECT3DVERTEXDECLARATION9 m_pTorusVertexDeclaration = NULL;  // vertex shader input data type
LPD3DXCONSTANTTABLE          m_pTorusVertexConstants   = NULL;  // array of vertex shader constants

LPDIRECT3DPIXELSHADER9       m_pPixelShader       = NULL;  // pixel shader
LPD3DXCONSTANTTABLE          m_pPixelConstants    = NULL;  // array of pixel shader constant

LPDIRECT3DPIXELSHADER9       m_pTorusPixelShader       = NULL;  // pixel shader
LPD3DXCONSTANTTABLE          m_pTorusPixelConstants    = NULL;  // array of pixel shader constant


D3DXMATRIXA16 matView;
D3DXMATRIXA16 matWorld;    
D3DXMATRIXA16 matProj;    
D3DXVECTOR3   vEyePt( 0.5f, 0.7f,-1.2f );
D3DXMATRIX  MFull,TMFull; // Full = World * View * Projection


D3DXVECTOR4 LightPos;                      //position of the light in world space
const D3DXVECTOR3 center(0.5f,0.9f,0.5f);  // center of the circle for the path of the light
const float rayon = 0.3f;                  // radius of this circle
float angle = 0.0f;                        // current angle
float angle_step = 0.004f;                  // step

// coeff for distance attenuation for lithing
const float kC = 0.1f;
const float kL = 0.5f;
const float kQ = 1.0f;
// coeff diffuse
float kD = 1.0f;
//coeff ambient
float kA = 0.0f;


const D3DXVECTOR3 TorusCenter(0.5f,0.35f,0.5f); // position of the torus
// param of the torus
const unsigned int TORUS_Nr = 20;  
const unsigned int TORUS_NR = 30;

// A structure for our custom vertex type
struct CUSTOMVERTEX
{
    FLOAT x, y, z;      // The untransformed, 3D position for the vertex	
	FLOAT nx,ny,nz;	    // the normal for ligthing (per vertex)	
	FLOAT tx,ty,tz;     // tangent
	FLOAT tu, tv;       // The texture coordinates
	FLOAT ntu, ntv;     // The texture coordinates in the normal map
};

// structure of vertex for the torus
struct TORUS_VERTEX
{
	FLOAT s,t;          // coordonnes sur le torus
};


//-----------------------------------------------------------------------------
// Name: InitTorusShaders()
// Desc: Initialize vertex shaders for the torus
//-----------------------------------------------------------------------------
HRESULT InitTorusShaders()
{
	//--------------------------- creation du vertex shader
	DWORD dwFlags = 0; 
    dwFlags |= D3DXSHADER_DEBUG;
    LPD3DXBUFFER pCode   = NULL;
    LPD3DXBUFFER pErrMsg = NULL;
	HRESULT       s;

	// array describing the data type :	
	D3DVERTEXELEMENT9 decl[] =
    {
      { 0,  0, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },      
	    D3DDECL_END()
    };

	// the type of data passed to the vertex shader is created
    s = g_pd3dDevice->CreateVertexDeclaration(decl,&m_pTorusVertexDeclaration);
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexDeclaration for the torus failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }

	// compile the HLSL source
	s=D3DXCompileShaderFromFile("vs_torus.cg",NULL,NULL,"main","vs_2_0",dwFlags,&pCode,&pErrMsg,&m_pTorusVertexConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}

	// create the shader from the compiled code
	char* szCode = (char*)pCode->GetBufferPointer();
    s = g_pd3dDevice->CreateVertexShader((DWORD*)pCode->GetBufferPointer(),&m_pTorusVertexShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexShader torus failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;		
    }

	//------------------ creation du pixel shader
	// compile the source code
	pErrMsg = NULL;
    s = D3DXCompileShaderFromFile("ps_torus.cg",NULL,NULL,"main","ps_2_0",dwFlags,&pCode,&pErrMsg,&m_pTorusPixelConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}    

	// create the pixel shader from the copiled code
    //szCode = (char*)pCode->GetBufferPointer();
    s  = g_pd3dDevice->CreatePixelShader((DWORD*)pCode->GetBufferPointer(),&m_pTorusPixelShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreatePixelShader torus failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }

	return S_OK;

}


//-----------------------------------------------------------------------------
// Name: InitShaders()
// Desc: Initialize vertex and pixel shaders
//-----------------------------------------------------------------------------
HRESULT InitShaders()
{
	//--------------------------- creation du vertex shader
	DWORD dwFlags = 0; 
    dwFlags |= D3DXSHADER_DEBUG;
    LPD3DXBUFFER pCode   = NULL;
    LPD3DXBUFFER pErrMsg = NULL;
	HRESULT       s;

	// array describing the data type :
	D3DVERTEXELEMENT9 decl[] =
    {
      { 0,  0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },      
	  { 0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },
	  { 0, 24, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },	  
	  { 0, 36, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 1 },	  
	  { 0, 44, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 2 },	  
        D3DDECL_END()
    };

	// the type of data passed to the vertex shader is created
    s = g_pd3dDevice->CreateVertexDeclaration(decl,&m_pVertexDeclaration);
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexDeclaration failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }

	// compile the HLSL source
	s=D3DXCompileShaderFromFile("vs_pass.cg",NULL,NULL,"main","vs_2_0",dwFlags,&pCode,&pErrMsg,&m_pVertexConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}

	// create the shader from the compiled code
	char* szCode = (char*)pCode->GetBufferPointer();
    s = g_pd3dDevice->CreateVertexShader((DWORD*)pCode->GetBufferPointer(),&m_pVertexShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreateVertexShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return E_FAIL;		
    }

	//------------------ creation du pixel shader
	// compile the source code
	pErrMsg = NULL;
    s = D3DXCompileShaderFromFile("ps_pass.cg",NULL,NULL,"main","ps_2_0",dwFlags,&pCode,&pErrMsg,&m_pPixelConstants);
	if( FAILED(s))
	{
		if (pErrMsg != NULL) MessageBox(NULL, (char *)(pErrMsg->GetBufferPointer()), "HLSL compilation error", MB_OK);
			else MessageBox(NULL,DXGetErrorString9(s), "HLSL compilation error", MB_OK);
        return E_FAIL;
	}    

	// create the pixel shader from the copiled code
    //szCode = (char*)pCode->GetBufferPointer();
    s  = g_pd3dDevice->CreatePixelShader((DWORD*)pCode->GetBufferPointer(),&m_pPixelShader);
    pCode->Release();
    if(FAILED(s))
    {
		MessageBox(NULL,"CreatePixelShader failed","CRendererDX9::Create",MB_OK|MB_ICONEXCLAMATION);
		return false;
    }	
	
	if( FAILED(InitTorusShaders() ))
		return E_FAIL;



	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: InitD3D()
// Desc: Initializes Direct3D
//-----------------------------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
    // Create the D3D object.
    if( NULL == ( g_pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
        return E_FAIL;

    // Set up the structure used to create the D3DDevice
    D3DPRESENT_PARAMETERS d3dpp;
    ZeroMemory( &d3dpp, sizeof(d3dpp) );
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;
	d3dpp.BackBufferCount = 1;
	d3dpp.hDeviceWindow = hWnd;
	d3dpp.EnableAutoDepthStencil = TRUE;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;

    // Create the D3DDevice
    if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd,
                                      D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                      &d3dpp, &g_pd3dDevice ) ) )
    {
		MessageBox(NULL, "Could not create the device", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Turn off culling, so we see the front and back of the triangle
    g_pd3dDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );

    // Turn off D3D lighting, since we are providing our own vertex colors
    g_pd3dDevice->SetRenderState( D3DRS_LIGHTING, FALSE );	
	
	
	if( FAILED(InitShaders() ))
		return E_FAIL;

	InitTimer();
    if FAILED(InitFont()) 
		return E_FAIL;

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: Init_Torus
// Desc: Create the vertex and index buffer of the orus()
//-----------------------------------------------------------------------------
HRESULT Init_Torus( int nr , int nR)
{	
	// Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( nr*(nR+1)*2*sizeof(TORUS_VERTEX),
                                                  0, 0,  
                                                  D3DPOOL_MANAGED, &g_pVB_Torus, NULL ) ) )
    {
		MessageBox(NULL, "Could not create vertex buffer", "Textures.exe", MB_OK);
        return E_FAIL;
    }

	// Fill the vertex buffer.
    TORUS_VERTEX* pVertices;
    if( FAILED( g_pVB_Torus->Lock( 0, 0, (VOID**)&pVertices, 0 ) ) )
	{
		MessageBox(NULL, "Could not lock", "Textures.exe", MB_OK);
        return E_FAIL;
	}	
	
	for(int r=0;r<nr;r++)		// sur le tube
		for(int R=0;R<nR+1;R++)		// sur le grand cercle
		{			
			pVertices[2*(R+r*(nR+1))].s = (1.0f/nR)*R;
			pVertices[2*(R+r*(nR+1))].t = (1.0f/nr)*r;

			pVertices[2*(R+r*(nR+1))+1].s = (1.0f/nR)*R;
			pVertices[2*(R+r*(nR+1))+1].t = (1.0f/nr)*(r+1);
		} 

    g_pVB_Torus->Unlock();
	return S_OK;
}

//-----------------------------------------------------------------------------
// Name: InitGeometry()
// Desc: Creates the scene geometry
//-----------------------------------------------------------------------------
HRESULT InitGeometry()
{
	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "metal.bmp", &g_pTexture ) ) )
    {
        MessageBox(NULL, "Could not find metal.bmp", "Textures.exe", MB_OK);        
		return E_FAIL;
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "cross.bmp", &g_pCrossTexture ) ) )
    {
        MessageBox(NULL, "Could not find cross.bmp", "Textures.exe", MB_OK);        
		return E_FAIL;
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "cross_normal.bmp", &g_pTorusNormalMap ) ) )
    {
        MessageBox(NULL, "Could not find cross_normal.bmp", "Textures.exe", MB_OK);        
		return E_FAIL;
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "floor.bmp", &g_pTextureFloor ) ) )
    {
        MessageBox(NULL, "Could not find chessboard.bmp", "Textures.exe", MB_OK);
		return E_FAIL;        
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "normal_map.bmp", &g_pWallsNormalMap ) ) )
    {
        MessageBox(NULL, "Could not find normal_map.bmp", "Textures.exe", MB_OK);
        return E_FAIL;        
    }

	// Use D3DX to create a texture from a file based image
    if( FAILED( D3DXCreateTextureFromFile( g_pd3dDevice, "normal_floor.bmp", &g_pFloorNormalMap ) ) )
    {
        MessageBox(NULL, "Could not find leaves_normal_map.bmp", "Textures.exe", MB_OK);
        return E_FAIL;        
    }

	
	CUSTOMVERTEX g_Vertices[] =
    {   // position           normal                            tangente              texture      normal_texture
        {  0.0f, 1.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 0.0f, 0.0f , 0.0f},  //1
        {  0.0f, 0.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //2        
		{  0.0f, 1.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //3
		
		{  0.0f, 1.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //4
        {  0.0f, 0.0f, 0.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //5        
		{  0.0f, 0.0f, 1.0f , 1.0f,0.0f,0.0f,                   0.0f,0.0f,1.0f,       1.0f , 1.0f, 1.0f , 1.0f},  //6

		{  0.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 0.0f, 0.0f , 0.0f},  //7		
		{  0.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //8
		{  1.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //9	

		{  1.0f, 1.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 0.0f, 1.0f , 0.0f},  //10
		{  0.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       0.0f , 1.0f, 0.0f , 1.0f},  //11
		{  1.0f, 0.0f, 1.0f , 0.0f,0.0f,-1.0f,                  1.0f,0.0f,0.0f,       1.0f , 1.0f, 1.0f , 1.0f},  //12		


		{  1.0f, 1.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 0.0f, 0.0f , 0.0f},  //13
		{  1.0f, 0.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 1.0f, 1.0f , 1.0f},  //14
		{  1.0f, 1.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 0.0f, 1.0f , 0.0f},  //15		

		{  1.0f, 0.0f, 0.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      1.0f , 1.0f, 1.0f , 1.0f},  //16
		{  1.0f, 1.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 0.0f, 0.0f , 0.0f},  //17
		{  1.0f, 0.0f, 1.0f , -1.0f,0.0f,0.0f,                  0.0f,0.0f,-1.0f,      0.0f , 1.0f, 0.0f , 1.0f},  //18		
		
    };
	
    // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 18*sizeof(CUSTOMVERTEX),
                                                  0, 0,  //on ne passe du format FVF : D3DFVF_CUSTOMVERTEX
                                                  D3DPOOL_MANAGED, &g_pVB_Walls, NULL ) ) )
    {
		MessageBox(NULL, "Could not create vertex buffer", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Fill the vertex buffer.
    VOID* pVertices;
    if( FAILED( g_pVB_Walls->Lock( 0, sizeof(g_Vertices), (void**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, g_Vertices, sizeof(g_Vertices) );
    g_pVB_Walls->Unlock();


	//----------------- FLOOR	
	// format is      x,y,z      normal                  tangente           texture      normal_texture
    CUSTOMVERTEX g_Vertices2[] =
    {
        {  0.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 1.0f , 0.0f , 1.0f  },  //1
        {  1.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 0.0f , 1.0f , 0.0f  },   //2
        {  0.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 0.0f , 0.0f , 0.0f },   //3

		{  0.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 0.0f , 1.0f , 0.0f , 1.0f  },  //4
        {  1.0f, 0.0f, 0.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 1.0f , 1.0f , 1.0f },   //5
        {  1.0f, 0.0f, 1.0f		, 0.0f , 1.0f , 0.0f	, 1.0f,0.0f,0.0f	, 1.0f , 0.0f , 1.0f , 0.0f  }    //6		
    };

    // Create the vertex buffer.
    if( FAILED( g_pd3dDevice->CreateVertexBuffer( 6*sizeof(CUSTOMVERTEX),
                                                  0, 0, // on se passe de D3DFVF_CUSTOMVERTEX,
                                                  D3DPOOL_MANAGED, &g_pVB_Floor, NULL ) ) )
    {
		MessageBox(NULL, "Could not create vertex buffer", "Textures.exe", MB_OK);
        return E_FAIL;
    }

    // Fill the vertex buffer.    
    if( FAILED( g_pVB_Floor->Lock( 0, sizeof(g_Vertices2), (void**)&pVertices, 0 ) ) )
        return E_FAIL;
    memcpy( pVertices, g_Vertices2, sizeof(g_Vertices2) );
    g_pVB_Floor->Unlock();

	Init_Torus(TORUS_Nr,TORUS_NR);

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: Cleanup()
// Desc: Releases all previously initialized objects
//-----------------------------------------------------------------------------
VOID Cleanup()
{	
	SAFE_RELEASE(g_pCrossTexture);
	SAFE_RELEASE(g_pTexture);
	SAFE_RELEASE(g_pTextureFloor);
	
	SAFE_RELEASE(g_pWallsNormalMap);
	SAFE_RELEASE(g_pFloorNormalMap);
	SAFE_RELEASE(g_pTorusNormalMap);	

	SAFE_RELEASE(m_pVertexShader);		
	SAFE_RELEASE(m_pVertexDeclaration);
	SAFE_RELEASE(m_pVertexConstants);

	SAFE_RELEASE(m_pTorusVertexShader);		
	SAFE_RELEASE(m_pTorusVertexDeclaration);
	SAFE_RELEASE(m_pTorusVertexConstants);

	SAFE_RELEASE(m_pPixelShader);
	SAFE_RELEASE(m_pPixelConstants);

	SAFE_RELEASE(m_pTorusPixelShader);
	SAFE_RELEASE(m_pTorusPixelConstants);

	SAFE_RELEASE(g_pVB_Walls);
	SAFE_RELEASE(g_pVB_Floor);
	SAFE_RELEASE(g_pVB_Torus);
	
	SAFE_RELEASE(g_pd3dDevice);
	SAFE_RELEASE(g_pD3D);	
}





//-----------------------------------------------------------------------------
// Name: SetupMatrices()
// Desc: Sets up the world, view, and projection transform Matrices.
//-----------------------------------------------------------------------------
VOID SetupMatrices()
{     
    // Set up our view matrix. A view matrix can be defined given an eye point,
    // a point to lookat, and a direction for which way is up. Here, we set the
    // eye five units back along the z-axis and up three units, look at the
    // origin, and define "up" to be in the y-direction.    
    D3DXVECTOR3 vLookatPt( 0.5f, 0.5f, 0.0f );
    D3DXVECTOR3 vUpVec( 0.0f, 1.0f, 0.0f );    
    D3DXMatrixLookAtLH( &matView, &vEyePt, &vLookatPt, &vUpVec );    

    // For the projection matrix, we set up a perspective transform (which
    // transforms geometry from 3D view space to 2D viewport space, with
    // a perspective divide making objects smaller in the distance). To build
    // a perpsective transform, we need the field of view (1/4 pi is common),
    // the aspect ratio, and the near and far clipping planes (which define at
    // what distances geometry should be no longer be rendered).
    D3DXMATRIXA16 matProj;
    D3DXMatrixPerspectiveFovLH( &matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f );    
	
	MFull = matWorld * matView * matProj;
	D3DXMatrixTranspose(&TMFull,&MFull);
}


//-----------------------------------------------------------------------------
// Name: Draw_Torus()
// Desc: Draws the torus
//-----------------------------------------------------------------------------
void Draw_Torus()
{
	D3DXMATRIXA16 T;
	D3DXMATRIXA16 Ry;

		
	D3DXMatrixTranslation(&T,TorusCenter.x,TorusCenter.y,TorusCenter.z);		
	D3DXMatrixRotationX(&matWorld,angle);
	D3DXMatrixRotationY(&Ry,angle/2);

	matWorld = matWorld*Ry*T;	

	D3DXMATRIXA16 invR;
	D3DXMATRIXA16 invRy;
	D3DXMATRIXA16 invT;
	D3DXMatrixRotationX(&invR,-angle);
	D3DXMatrixRotationY(&invRy,-angle/2);
	D3DXMatrixTranslation(&invT,-TorusCenter.x,-TorusCenter.y,-TorusCenter.z);		
	D3DXMATRIXA16 invWorld;
	invWorld=invT*invRy*invR;
	/*		
	
	float det=D3DXMatrixDeterminant(&matWorld);
	D3DXMatrixInverse(&invWorld,&det,&matWorld);
	*/
	D3DXVECTOR4 objectLightPos;
	D3DXVec4Transform(&objectLightPos,&LightPos,&invWorld);
	D3DXVECTOR4 objectEyePos;
	D3DXVec3Transform(&objectEyePos,&vEyePt,&invWorld);

	SetupMatrices();	


	HRESULT hr;
	hr = g_pd3dDevice->SetVertexDeclaration(m_pTorusVertexDeclaration);
	g_pd3dDevice->SetVertexShader     (m_pTorusVertexShader);
	//g_pd3dDevice->SetPixelShader      (m_pTorusPixelShader);

	
	D3DXHANDLE handle;
    if(handle = m_pTorusVertexConstants->GetConstantByName(NULL,"modelViewProj"))
			m_pTorusVertexConstants->SetMatrix(g_pd3dDevice,handle,(D3DXMATRIX*)&TMFull);
	else
		assert(0);

	if(handle = m_pTorusVertexConstants->GetConstantByName(NULL,"objectLightPosition"))
			m_pTorusVertexConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&objectLightPos);
	else
		assert(0);

		


	int TextureIndex=0;
	if(m_pPixelConstants && (handle = m_pPixelConstants->GetConstantByName(NULL,"decal")))
		{
			D3DXCONSTANT_DESC constDesc;
			UINT count = 1;
			m_pPixelConstants->GetConstantDesc(handle,&constDesc,&count);

			if(constDesc.RegisterSet == D3DXRS_SAMPLER)
			{
				TextureIndex = constDesc.RegisterIndex;
					
			}
			g_pd3dDevice->SetTexture( TextureIndex, g_pCrossTexture );
			
		}

	if(m_pPixelConstants && (handle = m_pPixelConstants->GetConstantByName(NULL,"bump")))
			{
				
				D3DXCONSTANT_DESC constDesc;
				UINT count = 1;
				m_pPixelConstants->GetConstantDesc(handle,&constDesc,&count);

				if(constDesc.RegisterSet == D3DXRS_SAMPLER)
				{
					TextureIndex = constDesc.RegisterIndex;				
					
				}
				g_pd3dDevice->SetTexture( TextureIndex, g_pTorusNormalMap );
			}

	kD = 5.0f;
	if(handle = m_pPixelConstants->GetConstantByName(NULL,"kD"))
		m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kD);
	
	kA = 0.5f;
	if(handle = m_pPixelConstants->GetConstantByName(NULL,"kA"))
		m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kA);	
	
		

	g_pd3dDevice->SetStreamSource( 0, g_pVB_Torus, 0, sizeof(TORUS_VERTEX) ); 
	for(int t=0 ; t<TORUS_Nr;t++)
		g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, t*(TORUS_NR+1)*2, TORUS_NR*2 );
}



//-----------------------------------------------------------------------------
// Name: Draw_Walls_And_Floor()
// Desc: Draws the walls and the floor (texture & bump mapping)
//-----------------------------------------------------------------------------
void Draw_Walls_And_Floor()
{
		D3DXMatrixIdentity(&matWorld);
		// Setup the world, view, and projection Matrices
        SetupMatrices();	

		g_pd3dDevice->SetVertexDeclaration(m_pVertexDeclaration);
		g_pd3dDevice->SetVertexShader     (m_pVertexShader);
		g_pd3dDevice->SetPixelShader      (m_pPixelShader);

	    // set the constant parameter modelViewProj matrix for the vertrex shader
		D3DXHANDLE handle;
        if(handle = m_pVertexConstants->GetConstantByName(NULL,"modelViewProj"))
			m_pVertexConstants->SetMatrix(g_pd3dDevice,handle,(D3DXMATRIX*)&TMFull);
		if(handle = m_pVertexConstants->GetConstantByName(NULL,"lightPos"))
			m_pVertexConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);		

		if(handle = m_pPixelConstants->GetConstantByName(NULL,"lightPos"))
			m_pPixelConstants->SetVector(g_pd3dDevice,handle,(D3DXVECTOR4*)&LightPos);

		if(handle = m_pPixelConstants->GetConstantByName(NULL,"kC"))
			m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kC);

		if(handle = m_pPixelConstants->GetConstantByName(NULL,"kL"))
			m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kL);

		if(handle = m_pPixelConstants->GetConstantByName(NULL,"kQ"))
			m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kQ);	
		
		kD = 1.0f;
		if(handle = m_pPixelConstants->GetConstantByName(NULL,"kD"))
			m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kD);	

		kA = 0.0f;
		if(handle = m_pPixelConstants->GetConstantByName(NULL,"kA"))
			m_pPixelConstants->SetFloat(g_pd3dDevice,handle,kA);	

		int TextureIndex=0;

		if(m_pPixelConstants && (handle = m_pPixelConstants->GetConstantByName(NULL,"decal")))
		{
			D3DXCONSTANT_DESC constDesc;
			UINT count = 1;
			m_pPixelConstants->GetConstantDesc(handle,&constDesc,&count);

			if(constDesc.RegisterSet == D3DXRS_SAMPLER)
			{
				TextureIndex = constDesc.RegisterIndex;
					
			}
			g_pd3dDevice->SetTexture( TextureIndex, g_pTexture );
			
		}
		
		

		if(m_pPixelConstants && (handle = m_pPixelConstants->GetConstantByName(NULL,"bump")))
			{
				
				D3DXCONSTANT_DESC constDesc;
				UINT count = 1;
				m_pPixelConstants->GetConstantDesc(handle,&constDesc,&count);

				if(constDesc.RegisterSet == D3DXRS_SAMPLER)
				{
					TextureIndex = constDesc.RegisterIndex;				
					
				}
				g_pd3dDevice->SetTexture( TextureIndex, g_pWallsNormalMap );
			}				        
		
		
		// render the walls
        g_pd3dDevice->SetStreamSource( 0, g_pVB_Walls, 0, sizeof(CUSTOMVERTEX) );        
        g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 6 );				
		// render the floor	
		g_pd3dDevice->SetTexture( TextureIndex, g_pFloorNormalMap );
        g_pd3dDevice->SetStreamSource( 0, g_pVB_Floor, 0, sizeof(CUSTOMVERTEX) );        
		g_pd3dDevice->SetTexture( 0, g_pTextureFloor );
        g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLELIST, 0, 2 );		
		
}

//-----------------------------------------------------------------------------
// Name: Render()
// Desc: Draws the scene
//-----------------------------------------------------------------------------
VOID Render()
{
	
    // Clear the backbuffer to a black color
    g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET| D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,0), 1.0f, 0 );
	
	ctime = timeGetTime();

    // Begin the scene
    if( SUCCEEDED( g_pd3dDevice->BeginScene() ) )
    {	
		angle+=angle_step;
		LightPos.x = (float) (center.x + rayon*cos(angle));
		LightPos.y = center.y;
		LightPos.z = (float) (center.z + rayon*sin(angle));
		LightPos.w = 1.0f;				
				
		Draw_Walls_And_Floor();				
		
		Draw_Torus();
		drawFPS(0,white);		

        // End the scene
        g_pd3dDevice->EndScene();
    }

    // Present the backbuffer contents to the display
    g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}




//-----------------------------------------------------------------------------
// Name: MsgProc()
// Desc: The window's message handler
//-----------------------------------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
            Cleanup();
            PostQuitMessage( 0 );
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}




//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: The application's entry point
//-----------------------------------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT )
{
    // Register the window class
    WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L,
                      GetModuleHandle(NULL), NULL, NULL, NULL, NULL,
                      "HLSL basics", NULL };
    RegisterClassEx( &wc );

    // Create the application's window
    HWND hWnd = CreateWindow( "HLSL basics", "HLSL basics : texture, lighting and bump mapping",
                              WS_OVERLAPPEDWINDOW, 100, 100, 800, 600,
                              GetDesktopWindow(), NULL, wc.hInstance, NULL );
	
    // Initialize Direct3D
    if( SUCCEEDED( InitD3D( hWnd ) ) )
    {
        // Create the scene geometry
        if( SUCCEEDED( InitGeometry() ) )
        {			
            // Show the window
            ShowWindow( hWnd, SW_SHOWDEFAULT );
            UpdateWindow( hWnd );
            // Enter the message loop
            MSG msg;
            ZeroMemory( &msg, sizeof(msg) );
            while( msg.message!=WM_QUIT )
            {
                if( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
                {
                    TranslateMessage( &msg );
                    DispatchMessage( &msg );
                }
                else
                    Render();
            }
        }
    }

    UnregisterClass( "D3D Tutorial", wc.hInstance );
    return 0;
}



